import json



def json_schema_primitive_to_avro_type(json_primitive: str, format: str, enum: list, field_name: str) -> str:
    """Convert a JSON-schema primitive type to Avro primitive type."""
    avro_primitive_map = {
        'string': 'string',
        'integer': 'int',
        'number': 'float',
        'boolean': 'boolean',
    }
    if json_primitive in avro_primitive_map:
        avro_primitive = avro_primitive_map[json_primitive]
    else:
        avro_primitive = json_primitive

    if format:
        if format in ('date-time', 'date'):
            avro_primitive = {'type': 'int', 'logicalType': 'date'}
        elif format in ('time'):
            avro_primitive = {'type': 'int', 'logicalType': 'time-millis'}
        elif format in ('duration'):
            avro_primitive = {'type': 'fixed', 'size': 12, 'logicalType': 'duration'}
        elif format in ('uuid'):
            avro_primitive = {'type': 'string', 'logicalType': 'uuid'}
    
    if enum:
        # replace white space with underscore
        enum = [e.replace(" ", "_") for e in enum]
        avro_primitive = {"type": "enum", "symbols": enum, "name": field_name + "_enum"}


    return avro_primitive
    

def merge_schemas(schemas):
    """Merge multiple Avro type schemas into one."""
    merged_schema = {}
    for schema in schemas:
        if 'type' not in merged_schema:
            merged_schema = schema
        else:
            if schema['type'] != merged_schema['type']:
                raise ValueError("Schema of different types cannot be merged")
            merged_schema['name'] = merged_schema['name'] + schema['name']
            merged_schema['fields'].extend(schema['fields'])
    return merged_schema


def json_type_to_avro_type(json_type: str | dict, field_name: str, namespace : str) -> dict:
    """Convert a JSON type to Avro type."""
    if isinstance(json_type, dict):
        t = json_type.get('type', 'object')
        if t == 'array':
            avro_type = {"type": "array", "items": json_type_to_avro_type(json_type['items'], field_name, namespace)}
        elif 'oneOf' in json_type:
            avro_type = [json_type_to_avro_type(one_type, field_name, namespace) for one_type in json_type['oneOf']]
        elif 'allOf' in json_type:
            avro_type = merge_schemas([json_type_to_avro_type(schema, field_name, namespace) for schema in json_type['allOf']])
        elif t == 'object':
            avro_type = json_schema_object_to_avro_record(json_type, namespace)
        else:
            avro_type = json_schema_primitive_to_avro_type(t, None, None, field_name)
    else:
        avro_type = json_schema_primitive_to_avro_type(json_type, None, None, field_name)
    return avro_type

def json_schema_object_to_avro_record(json_object: dict, namespace) -> dict:
    """Convert a JSON schema object declaration to an Avro record."""
    title = json_object.get('title')
    avro_record = {'type': 'record', 'fields': [], 'namespace': namespace}
    if title is not None:
        avro_record['name'] = title
    else:
        avro_record['name'] = "record"

    required_fields = json_object.get('required', [])
    if 'properties' in json_object:
        for field_name, field in json_object['properties'].items():
            avro_field_type = json_type_to_avro_type(field, field_name, namespace)
                
            if avro_field_type is None:
                raise ValueError(f"avro_field_type is None for field {field_name}")
            
            if not field_name in required_fields:
                avro_field = {"name": field_name, "type": ["null", avro_field_type]}
            else:
                avro_field = {"name": field_name, "type": avro_field_type}
            avro_record["fields"].append(avro_field)
    else:
        avro_record = {
            "type": "map",
            "values": [
                        "null",
                        "boolean",
                        "double",
                        "string"
                ]
            }       

    return avro_record
    

def jsons_to_avro(json_schema: dict | list, namespace: str) -> list:
    """Convert a JSON-schema to an Avro-schema."""
    avro_schema = []

    # check whether this is indeed a swagger file and then grab the definitions section
    if 'swagger' in json_schema:
        json_schema = json_schema.get('definitions', {})
        if not json_schema:
            raise ValueError('No definitions found in swagger file')
        for schema_name, schema in json_schema.items():
            avro_schema_item = json_schema_object_to_avro_record(schema, namespace)
            avro_schema_item['name'] = schema_name
            avro_schema.append(avro_schema_item)
        return avro_schema
    else:
        if not isinstance(json_schema, list):
            json_schema = [json_schema]

        for schema in json_schema:
            if schema['type'] == 'object':
                avro_schema.append(json_schema_object_to_avro_record(schema, namespace))
        return avro_schema
    
def convert_jsons_to_avro(json_schema_file_path: str, avro_schema_path: str, namespace: str = None) -> list:
    """Convert JSON schema file to Avro schema file."""
    with open(json_schema_file_path, 'r') as schema_file:
        json_schema = json.load(schema_file)
    avro_schema = jsons_to_avro(json_schema, namespace)
    with open(avro_schema_path, 'w') as avro_file:
        json.dump(avro_schema, avro_file, indent=4, sort_keys=True)
    return avro_schema


import json
import argparse

indent = '  '

def avro_primitive_to_proto_type(avro_type):
    """Map Avro primitive types to Protobuf types."""
    mapping = {
        'null': 'google.protobuf.Empty',  # Special handling may be required
        'boolean': 'bool',
        'int': 'int32',
        'long': 'int64',
        'float': 'float',
        'double': 'double',
        'bytes': 'bytes',
        'string': 'string',
    }
    # logical types require special handling
    if isinstance(avro_type, dict) and 'logicalType' in avro_type:
        logical_type = avro_type['logicalType']
        if logical_type == 'date':
            return 'string'
        elif logical_type == 'time-millis':
            return 'string'
        elif logical_type == 'timestamp-millis':
            return 'string'
        elif logical_type == 'decimal':
            precision = avro_type['precision']
            scale = avro_type['scale']
            return 'string'
        elif logical_type == 'duration':
            return 'string'
        elif logical_type == 'uuid':
            return 'string'
        
    return mapping.get(avro_type, avro_type)  # Default to string for unmapped types

def convert_field(avro_field, index):
    """Convert an Avro field to a Protobuf field."""
    field_type = avro_field['type']
    comment = ''
    if 'doc' in avro_field:
        comment = f'  // {avro_field["doc"]}\n{indent}'

    label = ''
    if isinstance(field_type, list) and 'null' in field_type:
        label = 'optional '
    if isinstance(field_type, list):
        # Handling union types (including nullable fields)
        non_null_types = [t for t in field_type if t != 'null']
        if len(non_null_types) == 1:
            field_type = non_null_types[0]
        else:
            # More complex unions may require manual handling or decisions
            field_type = 'string'  # Simplification for example purposes

    if isinstance(field_type, dict):
        # Nested types (e.g., records, enums) require special handling
        if field_type['type'] == 'record':
            return f'{indent}{comment}{label}message {field_type["name"]} {{\n' + "\n".join([convert_field(f, i+1) for i, f in enumerate(field_type['fields'])]) + '\n}\n'
        elif field_type['type'] == 'enum':
            return f'{indent}{comment}{label}enum {field_type["name"]} {{\n' + "\n".join([f'  {symbol} = {i};' for i, symbol in enumerate(field_type['symbols'])]) + '\n}\n'
        elif field_type['type'] == 'array':
            item_type = avro_primitive_to_proto_type(field_type['items'])
            return f'{indent}{comment}{label}repeated {item_type} {avro_field["name"]} = {index};'
        elif field_type['type'] == 'map':
            value_type = avro_primitive_to_proto_type(field_type['values'])
            return f'{indent}{comment}{label}map<string, {value_type}> {avro_field["name"]} = {index};'
        
    else:
        proto_type = avro_primitive_to_proto_type(field_type)
        return f'{indent}{comment}{label}{proto_type} {avro_field["name"]} = {index};'

def avro_schema_to_proto_messages(avro_schema_input):
    """Convert an Avro schema to Protobuf message definitions."""
    if not isinstance(avro_schema_input, list):
        avro_schema_list = [avro_schema_input]
    else:
        avro_schema_list = avro_schema_input
    
    proto_messages = []
    for avro_schema in avro_schema_list:
        comment = ''
        if 'doc' in avro_schema:
            comment = f'// {avro_schema["doc"]}\n'
        if avro_schema['type'] == 'record':
            message_name = avro_schema['name']
            proto_fields = [convert_field(field, i+1) for i, field in enumerate(avro_schema['fields'])]
            proto_messages.append(f'{comment}message {message_name} {{\n' + "\n".join(proto_fields) + '\n}')
        elif avro_schema['type'] == 'enum':
            enum_name = avro_schema['name']
            enum_values = "\n".join([f'  {symbol} = {i};' for i, symbol in enumerate(avro_schema['symbols'])])
            proto_messages.append(f'{comment}enum {enum_name} {{\n' + enum_values + '\n}')
        elif avro_schema['type'] == 'array':
            item_type = avro_primitive_to_proto_type(avro_schema['items'])
            proto_messages.append(f'{comment}repeated {item_type} {avro_schema["name"]} = 1;')
        elif avro_schema['type'] == 'map':
            value_type = avro_primitive_to_proto_type(avro_schema['values'])
            proto_messages.append(f'{comment}map<string, {value_type}> {avro_schema["name"]} = 1;')
    return "\n".join(proto_messages)

def convert_avro_to_proto(avro_schema_path, proto_file_path):
    """Convert Avro schema file to Protobuf .proto file."""
    with open(avro_schema_path, 'r') as avro_file:
        avro_schema = json.load(avro_file)
    proto_schema = 'syntax = "proto3";\n\n' + avro_schema_to_proto_messages(avro_schema)
    with open(proto_file_path, 'w') as proto_file:
        proto_file.write(proto_schema)




import argparse

from avrotize.avrotoproto import convert_avro_to_proto
from avrotize.jsonstoavro import convert_jsons_to_avro
from avrotize.prototoavro import convert_proto_to_avro


def main():
    parser = argparse.ArgumentParser(description='Convert Proto schema to Avro Schema')

    subparsers = parser.add_subparsers(dest='command')
    a2p_parser = subparsers.add_parser('p2a', help='Convert Avro schema to Proto schema')
    a2p_parser.add_argument('--proto', type=str, help='Path of the proto file', required=True)
    a2p_parser.add_argument('--avsc', type=str, help='Path to the Avro schema file', required=True)

    p2a_parser = subparsers.add_parser('a2p', help='Convert Proto schema to Avro schema')
    p2a_parser.add_argument('--proto', type=str, help='Path to the Proto file', required=True)
    p2a_parser.add_argument('--avsc', type=str, help='Path to the Avro schema file', required=True)

    j2a_parser = subparsers.add_parser('j2a', help='Convert JSON schema to Avro schema')
    j2a_parser.add_argument('--jsons', type=str, help='Path to the JSON schema file', required=True)
    j2a_parser.add_argument('--avsc', type=str, help='Path to the Avro schema file', required=True)
    j2a_parser.add_argument('--namespace', type=str, help='Namespace for the Avro schema', required=False)

    args = parser.parse_args()

    if args.command == 'p2a':
        avro_schema_path = args.avsc
        proto_file_path = args.proto
        convert_proto_to_avro(proto_file_path, avro_schema_path)
    elif args.command == 'a2p':
        proto_schema_path = args.proto
        avro_file_path = args.avsc
        convert_avro_to_proto(avro_file_path, proto_schema_path)
    elif args.command == 'j2a':
        json_schema_file_path = args.jsons
        avro_schema_path = args.avsc
        namespace = args.namespace
        convert_jsons_to_avro(json_schema_file_path, avro_schema_path, namespace=namespace)

if __name__ == "__main__":
    main()
